; OVLADANI NASTENNYCH DIGITALNICH HODIN
; 4 cislice (hodiny a  minuty) + 60 LED (po 1 sekunde)
; Zobrazovani sekund 3 rezimy:
;    Z1 - kazdou sek. rozsviti 1 LED, pak vsechny zhasne a to stale dokola
;    Z2 - 1 LED krouzi stale dokola
;    Z3 - zacatek jako Z1, pak je zase postupne zhasina               
; Upravil:  Durchy  (www.DurchanekJaroslav@seznam.cz)
; Original: Jarek Paluszynski (jarek.paluszynski[at]e-ware.pl)
; ATMEL 89C2051; 10MHz; 4x 4543; 8x 74HC164; NE 555
; Xtal 10,000 MHz.....1,2us/1 stroj. cyklus

;***** Mod prekladace ASM ********************************************
	$mod51

;**** Definice nazvu *************************************************  
  LED_D   BIT P3.0     ;data sekund
  CLOCK   BIT P3.1     ;posuv sekund
  TL_SET  BIT P3.2     ;tlacitko nastaveni
  TL_UP   BIT P3.3     ;tlacitko prirustku +1
  SP_Z2   BIT P3.4     ;spinac rezimu 2
  SP_Z3   BIT P3.5     ;spinac rezimu 3
  BUZZ    BIT P3.7     ;ovladani buzzeru
  
;***** Prerusovaci system ******************************************** 
	ORG 0000H
	LJMP START
	ORG 0003h		          ;INT0/SET (nastaveni)
	LJMP PK1
	ORG 000Bh		          ;T0/casovac (chod hodin)
	LJMP TIMER 
	ORG 0013h	            ;INT1/UP  (zvyseni o 1)
	LJMP PK2
  	
;***** Zacatek programu **********************************************
START: CLR BUZZ	        ;vypni Alarm  (0)
	SETB CLOCK	          ;nastav Clock (1)
	SETB LED_D	          ;nastav Data  (1)
	SETB TL_SET	          ;nastav INT0 - vstup
	SETB TL_UP	          ;nastav INT1 - vstup
	CLR TCON.0	          ;INT0 sestupna 
	CLR TCON.2	          ;INT1 sestupna
	MOV P1,#00000000b     ;zhasni displej
	MOV R0,#0	            ;pomocny pro sekundy
	MOV R1,#0	            ;sekundy
	MOV R2,#0	            ;minuty
	MOV R3,#0	            ;hodiny
	MOV R4,#0	            ;minuty pro alarm
	MOV R5,#0	            ;hodiny pro alarm
	MOV R6,#1	            ;zobrazeni
			                  ;moznosti zobrazeni (R6) 
			                  ;1 - chod hodin
			                  ;2 - nastaveni minut
			                  ;4 - nastaveni hodin
			                  ;8 - nastaveni minut alarmu
			                  ;16 - nastaveni hodin alarmu
			                  ;32 - nastaveni alarmu ON/OFF
	MOV R7,#0	            ;alarm:  0 - OFF
			                  ;      128 - ON
	MOV TMOD,#00010001b   ;c0/1  mod 1 (16b)
	MOV IE,#10000011b     ;preruseni povoleno
	MOV IP,#00000000b     ;nastaveni priority
	MOV TH0,#01111111b    ;------- 
	MOV TL0,#11111111b    ;(32767)
	SETB TCON.4	          ;TR0 - spustit c0
  
;************** Hlavni smycka programu********************************
loop:	SETB IE.0	        ;povol INT0  
	SETB IE.1	            ;povol c0
	LJMP loop	            ;hlavni smycka programu
  
;************** SET - nastaveni **************************************
PK1:	MOV B,#1
zpoz2:	MOV A,#1	      ;zpozdeni, zakmit tlacitka
zpoz1:	INC A           ;+ zapipani buzzeru
	JNZ zpoz1            
	INC B
	MOV A,B
	CJNE A,#200,g1
	SETB BUZZ            
g1:	JNZ zpoz2          
	CLR BUZZ
	CJNE R7,#128,k1       ;nastaveni alarmu
	MOV A,R2	            ;R2(aktual. minuty)
	XRL A,R4              ;R4(alarm minuty)  
	CJNE A,#0,k1	        ;NE
	MOV A,R3              ;R3(aktual. hodiny)
	XRL A,R5              ;R5(alarm hodiny)
	CJNE A,#0,k1	        ;NE
	CLR BUZZ	            ;vypni buzzer
	MOV R7,#0	            ;stav alarmu (OFF)
	RETI
  
;****** Moznosti zobrazeni *******************************************
k1:	MOV A,R6
	RL A
	MOV R6,A
	CJNE R6,#64,DALE   
	MOV R6,#1
	CLR IE.2	            ;zakaz INT1
	LJMP ZAPIS
  
;***** SET minut *****************************************************
DALE:	SETB IE.2	        ;povol INT1 
	CJNE R6,#2,SET_1      
	MOV A,R2	            ;minuty :10
	MOV B,#10
	DIV AB
	ADD A,#10000000b
	MOV P1,A             
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;minuty :1
	ADD A,#00100000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
; Poznamka: pokud posleme dekoderu 4543 - 15(1111b) 
; stav - naplneni, nic nezobrazi (zhasne displej)  
	MOV A,#15	            ;hodiny :10 zhasni 
	MOV B,#15            
	ADD A,#01000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;hodiny :1 zhasni 
	ADD A,#00010000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
  
;***** SET hodin *****************************************************
SET_1: CJNE R6,#4,SET_2 
	MOV A,#15	            ;minuty :10 zhasni 
	MOV B,#15
	ADD A,#10000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;minuty :1 zhasni 
	ADD A,#00100000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,R3	            ;hodiny :10
	MOV B,#10
	DIV AB
	ADD A,#01000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;hodiny :1
	ADD A,#00010000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
  
;***** SET Alarm minuty **********************************************
SET_2:	CJNE R6,#8,SET_3 
	MOV A,R4	            ;AL minuty :10
	MOV B,#10
	DIV AB
	ADD A,#10000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;AL minuty :1
	ADD A,#00100000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,#15	            ;AL hodiny :10 zhasni 
	MOV B,#15
	ADD A,#01000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;AL hodiny :1 zhasni 
	ADD A,#00010000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
  
;***** SET Alarm hodiny **********************************************
SET_3:	CJNE R6,#16,SET_4 
	MOV A,#15	            ;AL minuty :10 zhasni
	MOV B,#15
	ADD A,#10000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;AL minuty :1 zhasni
	ADD A,#00100000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,R5	            ;AL hodiny :10
	MOV B,#10
	DIV AB
	ADD A,#01000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;AL hodiny :1
	ADD A,#00010000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
  
;***** SET Alarm zobrazeni stavu *************************************
SET_4:	CJNE R6,#32,SET_5 
	MOV A,#15	            ;zhasni 2.displej
	ADD A,#10000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,R7	            ;stav alarmu
	MOV B,#128            ;porovnej
	DIV AB
	ADD A,#00100000b      ;zobraz stav, 1.displej
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,#15	            ;zhasni 4.displej
	MOV B,#15
	ADD A,#01000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
	MOV A,B 	            ;zhasni 3.displej
	ADD A,#00010000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
SET_5:	RETI

;************** UP - prirustek +1 *****************************************
PK2:	MOV B,#80	      
zpoz4:	MOV A,#1	      ;zpozdeni, zakmit tlacitka
zpoz3:	INC A           ;+ zapipani buzzeru
	JNZ zpoz3
	INC B
	MOV A,B
	CJNE A,#200,g2
	SETB BUZZ
g2:	JNZ zpoz4
	CLR BUZZ
  CJNE R6,#2,d1	        ;postup pricteni hodnoty
	INC R2
	CJNE R2,#60,d1        ;max. 60 minut
	MOV R2,#0
d1:	CJNE R6,#4,d2
	INC R3
	CJNE R3,#24,d2        ;max. 24 hodin
	MOV R3,#0
d2:	CJNE R6,#8,d3
	INC R4
	CJNE R4,#60,d3        ;max. 60 min. AL
	MOV R4,#0
d3:	CJNE R6,#16,d4
	INC R5
	CJNE R5,#24,d4        ;max. 24 hod. AL
	MOV R5,#0
d4:	CJNE R6,#32,d5      ;alarm ON/OFF
	MOV A,R7
	ADD A,#128
	MOV R7,A
d5:	LJMP DALE           ;Konec rucniho zastaveni 

;*************Obsluha preruseni T0************************************ 
TIMER:	CLR IE.0	      ;INT1 - zakazano
	CJNE R0,#16,w1
  MOV TH0,#01111110b	  
	MOV TL0,#01101010b    ;nast. 01111110 01101010  (32362)
	MOV R0,#0
	LJMP w2
w1:	MOV TH0,#00111100b	
	MOV TL0,#10101111b	  ;nast. 00111100 10101111  (15535)
	INC R0
  
;****** Vyhodnoceni alarmu *******************************************
w2:	CJNE R7,#128,k2 	  ;mame alarm
	MOV A,R2              ;aktualni stav minut
	XRL A,R4              ;porovnej
	CJNE A,#0,k2		      ;NE
	MOV A,R3              ;aktualni stav hodin
	XRL A,R5              ;porovnej
	CJNE A,#0,k2		      ;NE
	MOV A,#00000001b	    ;stav alarmu
	ANL A,R0              ;porovnej
	CJNE A,#00000001b,k3  ;NE
	CPL BUZZ              ;neguj buzzer
	LJMP k3
k2:	CLR BUZZ            ;vypni buzzer

;**** Rezimy zobrazeni sekund ****************************************
k3:	CJNE R0,#10,Navrat1 ;pokud R0=10, pak mame novou sekundu
  JNB SP_Z2,z2          ;rezim po 1 LED
  JNB SP_Z3,z3          ;rezim naplneni/vyprazdneni
; Rezim Z1 - rozsviti LED 1/1s,60/zhane a znova.....  
z1:	CLR LED_D		        ;log.0 na data
	CLR CLOCK		          ;log.0 na clock
	SETB CLOCK		        ;log.1 na clock (nabezna hrana)
	INC R1			          ;zvyseni sekund
	CJNE R1,#60,Navrat1 	;je-li R1=60, mame novou minutu
	MOV A,#100
	SETB LED_D		        ;log.1 na data
LED_R: CLR CLOCK		    ;log.0 na clock
	SETB CLOCK		        ;log.1 na clock (nabezna hrana)
	INC A
	JNZ LED_R             ;resetuj vsechny LED
	CLR LED_D		          ;prvni LED je gratis
	CLR CLOCK
	SETB CLOCK
  LJMP OK
  
;**** Prodlouzeni skoku **********************************************  
Navrat1: LJMP NAVRAT
;*********************************************************************  
; Rezim  Z2 - 1 LED krouzi stale dokola...
z2: CJNE R1,#59,x1      ;neni 59s, preskoc
  CLR LED_D             ;log.0 na data
  SJMP x2               ;preskoc
x1: SETB LED_D          ;log.1 na data  
x2: CLR CLOCK           ;log.0 na clock
  SETB CLOCK            ;clock (nabezna hrana)
  INC R1                ;zvyseni sekund
  CJNE R1,#60,NAVRAT    ;R1=60, mame novou minutu
  LJMP OK               ;pokracuj na min. a hod.
    
; Rezim Z3 - rozsviti LED 1/1s,zhane LED 1/1s a znova...
z3: CJNE R1,#59,y1      ;neni 59s, preskoc
  CPL LED_D             ;neguj data
y1: CLR CLOCK           ;log.0 na clock
  SETB CLOCK            ;clock (nabezna hrana)
  INC R1                ;zvyseni sekund
  CJNE R1,#60,NAVRAT    ;R1=60, mame novou minutu 
        
;**** Zmena stavu minut a hodin **************************************
OK:	MOV R1,#0           ;vynuluj sekundy
	INC R2			          ;zvyseni minut
	CJNE R2,#60,ZAPIS 	  ;znena minut
	MOV R2,#0             ;vynuluj minuty
	INC R3			          ;zvyseni hodiny
	CJNE R3,#24,ZAPIS     ;zmena hodin
	MOV R3,#0             ;vynuluj hodiny
  
;**** Zapis - nove data **********************************************
ZAPIS:	CJNE R6,#1,NAVRAT
	MOV A,R2		          ;nastav minuty :10
	MOV B,#10
	DIV AB
	ADD A,#10000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
  MOV A,B 		          ;nastav minuty :1
	ADD A,#00100000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
  MOV A,R3		          ;nastav desitky hodin
	MOV B,#10
	DIV AB
	CJNE A,#0,k4		      ;desitky hodin=0, pak nezobrazovat
	MOV A,#15
; posle kod BCD 1111 pro decoder desitek minut a ten ji vyhodnoti jako 
; preplneni a tak na displeji nezobrazi zadnou cislici !!  
k4:	ADD A,#01000000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
  MOV A,B 		          ;nastav hodiny :1
	ADD A,#00010000b
	MOV P1,A
	ANL A,#00001111b
	MOV P1,A
NAVRAT:	RETI			  
      END               ;konec programu